@@ -3,7 +3,7 @@
 
 /*
    Copyright (c) 2000, 2011, Oracle and/or its affiliates.
-   Copyright (c) 2009, 2019, MariaDB
+   Copyright (c) 2009, 2022, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,21 +35,21 @@ protected:
      character set. No memory is allocated.
      @retval A pointer to the str_value member.
    */
-  virtual String *make_empty_result()
+  virtual String *make_empty_result(String *str)
   {
     /*
       Reset string length to an empty string. We don't use str_value.set() as
       we don't want to free and potentially have to reallocate the buffer
       for each call.
     */
-    if (!str_value.is_alloced())
-      str_value.set("", 0, collation.collation); /* Avoid null ptrs */
+    if (!str->is_alloced())
+      str->set("", 0, collation.collation); /* Avoid null ptrs */
     else
     {
-      str_value.length(0);                      /* Reuse allocated area */
-      str_value.set_charset(collation.collation);
+      str->length(0);                      /* Reuse allocated area */
+      str->set_charset(collation.collation);
     }
-    return &str_value; 
+    return str;
   }
 public:
   Item_str_func(THD *thd): Item_func(thd) { decimals=NOT_FIXED_DEC; }
@@ -145,7 +145,7 @@ class Item_func_md5 :public Item_str_ascii_checksum_func
 public:
   Item_func_md5(THD *thd, Item *a): Item_str_ascii_checksum_func(thd, a) {}
   String *val_str_ascii(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     fix_length_and_charset(32, default_charset());
     return FALSE;
@@ -165,7 +165,7 @@ class Item_func_sha :public Item_str_ascii_checksum_func
 public:
   Item_func_sha(THD *thd, Item *a): Item_str_ascii_checksum_func(thd, a) {}
   String *val_str_ascii(String *) override;    
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("sha") };
@@ -181,7 +181,7 @@ public:
   Item_func_sha2(THD *thd, Item *a, Item *b)
    :Item_str_ascii_checksum_func(thd, a, b) {}
   String *val_str_ascii(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("sha2") };
@@ -198,7 +198,7 @@ public:
   Item_func_to_base64(THD *thd, Item *a)
    :Item_str_ascii_checksum_func(thd, a) {}
   String *val_str_ascii(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("to_base64") };
@@ -215,7 +215,7 @@ public:
   Item_func_from_base64(THD *thd, Item *a)
    :Item_str_binary_checksum_func(thd, a) { }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("from_base64") };
@@ -246,7 +246,7 @@ class Item_func_aes_encrypt :public Item_aes_crypt
 public:
   Item_func_aes_encrypt(THD *thd, Item *a, Item *b)
    :Item_aes_crypt(thd, a, b) {}
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("aes_encrypt") };
@@ -261,7 +261,7 @@ class Item_func_aes_decrypt :public Item_aes_crypt
 public:
   Item_func_aes_decrypt(THD *thd, Item *a, Item *b):
     Item_aes_crypt(thd, a, b) {}
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("aes_decrypt") };
@@ -282,7 +282,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("natural_sort_key")};
     return name;
   }
-  bool fix_length_and_dec(void) override;
+  bool fix_length_and_dec(THD *thd) override;
   Item *get_copy(THD *thd) override
   {
     return get_item_copy<Item_func_natural_sort_key>(thd, this);
@@ -308,7 +308,7 @@ public:
   Item_func_concat(THD *thd, List<Item> &list): Item_str_func(thd, list) {}
   Item_func_concat(THD *thd, Item *a, Item *b): Item_str_func(thd, a, b) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("concat") };
@@ -351,7 +351,7 @@ public:
   Item_func_decode_histogram(THD *thd, Item *a, Item *b):
     Item_str_func(thd, a, b) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(system_charset_info);
     max_length= MAX_BLOB_WIDTH;
@@ -373,7 +373,7 @@ class Item_func_concat_ws :public Item_str_func
 public:
   Item_func_concat_ws(THD *thd, List<Item> &list): Item_str_func(thd, list) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("concat_ws") };
@@ -390,7 +390,7 @@ class Item_func_reverse :public Item_str_func
 public:
   Item_func_reverse(THD *thd, Item *a): Item_str_func(thd, a) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("reverse") };
@@ -408,7 +408,7 @@ public:
   Item_func_replace(THD *thd, Item *org, Item *find, Item *replace):
     Item_str_func(thd, org, find, replace) {}
   String *val_str(String *to) override { return val_str_internal(to, NULL); };
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str_internal(String *str, String *empty_string_for_null);
   LEX_CSTRING func_name_cstring() const override
   {
@@ -456,7 +456,7 @@ public:
     DBUG_VOID_RETURN;
   }
   String *val_str(String *str) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("regexp_replace") };
@@ -481,7 +481,7 @@ public:
     DBUG_VOID_RETURN;
   }
   String *val_str(String *str) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("regexp_substr") };
@@ -499,7 +499,7 @@ public:
                    Item *new_str):
     Item_str_func(thd, org, start, length, new_str) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("insert") };
@@ -531,7 +531,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("lcase") };
     return name;
   }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_lcase>(thd, this); }
 };
@@ -545,7 +545,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("ucase") };
     return name;
   }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_ucase>(thd, this); }
 };
@@ -557,7 +557,7 @@ class Item_func_left :public Item_str_func
 public:
   Item_func_left(THD *thd, Item *a, Item *b): Item_str_func(thd, a, b) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("left") };
@@ -574,7 +574,7 @@ class Item_func_right :public Item_str_func
 public:
   Item_func_right(THD *thd, Item *a, Item *b): Item_str_func(thd, a, b) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("right") };
@@ -595,7 +595,7 @@ public:
   Item_func_substr(THD *thd, Item *a, Item *b, Item *c):
     Item_str_func(thd, a, b, c) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("substr") };
@@ -612,7 +612,7 @@ public:
   Item_func_sformat(THD *thd, List<Item> &list);
   ~Item_func_sformat() { delete [] val_arg; }
   String *val_str(String*) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("sformat") };
@@ -627,16 +627,16 @@ class Item_func_substr_oracle :public Item_func_substr
 protected:
   longlong get_position() override
   { longlong pos= args[1]->val_int(); return pos == 0 ? 1 : pos; }
-  String *make_empty_result() override
+  String *make_empty_result(String *str) override
   { null_value= 1; return NULL; }
 public:
   Item_func_substr_oracle(THD *thd, Item *a, Item *b):
     Item_func_substr(thd, a, b) {}
   Item_func_substr_oracle(THD *thd, Item *a, Item *b, Item *c):
     Item_func_substr(thd, a, b, c) {}
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    bool res= Item_func_substr::fix_length_and_dec();
+    bool res= Item_func_substr::fix_length_and_dec(thd);
     set_maybe_null();
     return res;
   }
@@ -656,7 +656,7 @@ public:
   Item_func_substr_index(THD *thd, Item *a,Item *b,Item *c):
     Item_str_func(thd, a, b, c) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("substring_index") };
@@ -676,7 +676,7 @@ protected:
   String *trimmed_value(String *res, uint32 offset, uint32 length)
   {
     if (length == 0)
-      return make_empty_result();
+      return make_empty_result(&tmp_value);
 
     tmp_value.set(*res, offset, length);
     /*
@@ -701,7 +701,7 @@ public:
   Item_func_trim(THD *thd, Item *a): Item_str_func(thd, a) {}
   Sql_mode_dependency value_depends_on_sql_mode() const override;
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("trim") };
@@ -717,7 +717,7 @@ public:
 class Item_func_trim_oracle :public Item_func_trim
 {
 protected:
-  String *make_empty_result() override
+  String *make_empty_result(String *str) override
   { null_value= 1; return NULL; }
   LEX_CSTRING func_name_ext() const override
   {
@@ -733,9 +733,9 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("trim_oracle") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    bool res= Item_func_trim::fix_length_and_dec();
+    bool res= Item_func_trim::fix_length_and_dec(thd);
     set_maybe_null();
     return res;
   }
@@ -769,7 +769,7 @@ public:
 class Item_func_ltrim_oracle :public Item_func_ltrim
 {
 protected:
-  String *make_empty_result() override
+  String *make_empty_result(String *str) override
   { null_value= 1; return NULL; }
   LEX_CSTRING func_name_ext() const override
   {
@@ -785,9 +785,9 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("ltrim_oracle") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    bool res= Item_func_ltrim::fix_length_and_dec();
+    bool res= Item_func_ltrim::fix_length_and_dec(thd);
     set_maybe_null();
     return res;
   }
@@ -817,7 +817,7 @@ public:
 class Item_func_rtrim_oracle :public Item_func_rtrim
 {
 protected:
-  String *make_empty_result() override
+  String *make_empty_result(String *str) override
   { null_value= 1; return NULL; }
   LEX_CSTRING func_name_ext() const override
   {
@@ -833,9 +833,9 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("rtrim_oracle") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    bool res= Item_func_rtrim::fix_length_and_dec();
+    bool res= Item_func_rtrim::fix_length_and_dec(thd);
     set_maybe_null();
     return res;
   }
@@ -866,7 +866,7 @@ public:
     Item_str_ascii_checksum_func(thd, a), alg(al), deflt(0) {}
   String *val_str_ascii(String *str) override;
   bool fix_fields(THD *thd, Item **ref) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     fix_length_and_charset((alg == 1 ?
                             SCRAMBLED_PASSWORD_CHAR_LENGTH :
@@ -897,7 +897,7 @@ public:
   Item_func_des_encrypt(THD *thd, Item *a, Item *b)
    :Item_str_binary_checksum_func(thd, a, b) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     set_maybe_null();
     /* 9 = MAX ((8- (arg_len % 8)) + 1) */
@@ -922,7 +922,7 @@ public:
   Item_func_des_decrypt(THD *thd, Item *a, Item *b)
    :Item_str_binary_checksum_func(thd, a, b) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     set_maybe_null();
     /* 9 = MAX ((8- (arg_len % 8)) + 1) */
@@ -966,7 +966,7 @@ public:
     constructor_helper();
   }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     set_maybe_null();
     max_length = 13;
@@ -999,7 +999,7 @@ public:
   Item_func_encode(THD *thd, Item *a, Item *seed_arg):
     Item_str_binary_checksum_func(thd, a, seed_arg) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("encode") };
@@ -1057,9 +1057,9 @@ class Item_func_database :public Item_func_sysconst
 public:
   Item_func_database(THD *thd): Item_func_sysconst(thd) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    max_length= MAX_FIELD_NAME * system_charset_info->mbmaxlen;
+    max_length= NAME_CHAR_LEN * system_charset_info->mbmaxlen;
     set_maybe_null();
     return FALSE;
   }
@@ -1091,7 +1091,7 @@ public:
   {
     str->append(func_name_cstring());
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= 512 * system_charset_info->mbmaxlen;
     null_value= false;
@@ -1119,7 +1119,7 @@ public:
     return (null_value ? 0 : &str_value);
   }
   bool fix_fields(THD *thd, Item **ref) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= (uint32) (username_char_length +
                  HOSTNAME_LENGTH + 1) * SYSTEM_CHARSET_MBMAXLEN;
@@ -1173,7 +1173,7 @@ public:
   Item_func_current_role(THD *thd, Name_resolution_context *context_arg):
     Item_func_sysconst(thd), context(context_arg) {}
   bool fix_fields(THD *thd, Item **ref) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= (uint32) username_char_length * SYSTEM_CHARSET_MBMAXLEN;
     return FALSE;
@@ -1209,7 +1209,7 @@ class Item_func_soundex :public Item_str_func
 public:
   Item_func_soundex(THD *thd, Item *a): Item_str_func(thd, a) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("soundex") };
@@ -1227,7 +1227,7 @@ public:
   double val_real() override;
   longlong val_int() override;
   String *val_str(String *str) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("elt") };
@@ -1245,7 +1245,7 @@ class Item_func_make_set :public Item_str_func
 public:
   Item_func_make_set(THD *thd, List<Item> &list): Item_str_func(thd, list) {}
   String *val_str(String *str) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("make_set") };
@@ -1266,7 +1266,7 @@ public:
     Item_str_ascii_func(thd, org, dec, lang) {}
 
   String *val_str_ascii(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("format") };
@@ -1290,7 +1290,7 @@ public:
   { collation.set(cs); }
   String *val_str(String *) override;
   void append_char(String * str, int32 num);
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= arg_count * 4;
     return FALSE;
@@ -1311,7 +1311,7 @@ public:
   Item_func_chr(THD *thd, Item *arg1, CHARSET_INFO *cs):
     Item_func_char(thd, arg1, cs) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= 4;
     return FALSE;
@@ -1332,7 +1332,7 @@ public:
   Item_func_repeat(THD *thd, Item *arg1, Item *arg2):
     Item_str_func(thd, arg1, arg2) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("repeat") };
@@ -1348,7 +1348,7 @@ class Item_func_space :public Item_str_func
 public:
   Item_func_space(THD *thd, Item *arg1): Item_str_func(thd, arg1) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("space") };
@@ -1365,7 +1365,7 @@ public:
   Item_func_binlog_gtid_pos(THD *thd, Item *arg1, Item *arg2):
     Item_str_func(thd, arg1, arg2) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("binlog_gtid_pos") };
@@ -1389,7 +1389,7 @@ public:
     Item_str_func(thd, arg1, arg2, arg3) {}
   Item_func_pad(THD *thd, Item *arg1, Item *arg2):
     Item_str_func(thd, arg1, arg2) {}
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
 };
 
 
@@ -1414,16 +1414,16 @@ public:
 
 class Item_func_rpad_oracle :public Item_func_rpad
 {
-  String *make_empty_result() override
+  String *make_empty_result(String *str) override
   { null_value= 1; return NULL; }
 public:
   Item_func_rpad_oracle(THD *thd, Item *arg1, Item *arg2, Item *arg3):
     Item_func_rpad(thd, arg1, arg2, arg3) {}
   Item_func_rpad_oracle(THD *thd, Item *arg1, Item *arg2):
     Item_func_rpad(thd, arg1, arg2) {}
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    bool res= Item_func_rpad::fix_length_and_dec();
+    bool res= Item_func_rpad::fix_length_and_dec(thd);
     set_maybe_null();
     return res;
   }
@@ -1457,16 +1457,16 @@ public:
 
 class Item_func_lpad_oracle :public Item_func_lpad
 {
-  String *make_empty_result() override
+  String *make_empty_result(String *str) override
   { null_value= 1; return NULL; }
 public:
   Item_func_lpad_oracle(THD *thd, Item *arg1, Item *arg2, Item *arg3):
     Item_func_lpad(thd, arg1, arg2, arg3) {}
   Item_func_lpad_oracle(THD *thd, Item *arg1, Item *arg2):
     Item_func_lpad(thd, arg1, arg2) {}
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    bool res= Item_func_lpad::fix_length_and_dec();
+    bool res= Item_func_lpad::fix_length_and_dec(thd);
     set_maybe_null();
     return res;
   }
@@ -1491,7 +1491,7 @@ public:
     return name;
   }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(default_charset());
     fix_char_length(64);
@@ -1530,7 +1530,7 @@ public:
     DBUG_ASSERT(fixed());
     return m_arg0_type_handler->Item_func_hex_val_str_ascii(this, str);
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(default_charset(), DERIVATION_COERCIBLE, MY_REPERTOIRE_ASCII);
     decimals=0;
@@ -1557,7 +1557,7 @@ public:
     return name;
   }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(&my_charset_bin);
     decimals=0;
@@ -1583,7 +1583,7 @@ public:
     set_maybe_null();
   }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(args[0]->collation);
     decimals=0;
@@ -1637,7 +1637,7 @@ public:
       tmp->set_charset(&my_charset_bin);
     return tmp;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(&my_charset_bin);
     max_length=args[0]->max_length;
@@ -1666,7 +1666,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("load_file") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(&my_charset_bin, DERIVATION_COERCIBLE);
     set_maybe_null();
@@ -1692,7 +1692,7 @@ class Item_func_export_set: public Item_str_func
   Item_func_export_set(THD *thd, Item *a, Item *b, Item* c, Item* d, Item* e):
     Item_str_func(thd, a, b, c, d, e) {}
   String  *val_str(String *str) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("export_set") };
@@ -1714,7 +1714,7 @@ public:
     return name;
   }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(args[0]->collation);
     ulonglong max_result_length= (ulonglong) args[0]->max_length * 2 +
@@ -1774,7 +1774,6 @@ public:
               (cs->mbmaxlen > 1 || !(cs->state & MY_CS_NONASCII))));
     }
   }
-  bool is_json_type() override { return args[0]->is_json_type(); }
   String *val_str(String *) override;
   longlong val_int() override
   {
@@ -1812,7 +1811,7 @@ public:
       return 1;
     return res;
   }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("convert") };
@@ -1831,7 +1830,7 @@ public:
   Item_func_set_collation(THD *thd, Item *a, CHARSET_INFO *set_collation):
     Item_str_func(thd, a), m_set_collation(set_collation) {}
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   bool eq(const Item *item, bool binary_cmp) const override;
   LEX_CSTRING func_name_cstring() const override
   {
@@ -1856,7 +1855,7 @@ class Item_func_expr_str_metadata :public Item_str_func
 {
 public:
   Item_func_expr_str_metadata(THD *thd, Item *a): Item_str_func(thd, a) { }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
      collation.set(system_charset_info);
      max_length= 64 * collation.collation->mbmaxlen; // should be enough
@@ -1924,7 +1923,7 @@ public:
     return name;
   }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   bool eq(const Item *item, bool binary_cmp) const override
   {
     if (!Item_str_func::eq(item, binary_cmp))
@@ -1945,17 +1944,29 @@ public:
 class Item_func_crc32 :public Item_long_func
 {
   bool check_arguments() const override
-  { return args[0]->check_type_can_return_str(func_name_cstring()); }
+  {
+    return args[0]->check_type_can_return_str(func_name_cstring()) &&
+      (arg_count == 1 ||
+       args[1]->check_type_can_return_int(func_name_cstring()));
+  }
   String value;
+  uint32 (*const crc_func)(uint32, const void*, size_t);
 public:
-  Item_func_crc32(THD *thd, Item *a): Item_long_func(thd, a)
+  Item_func_crc32(THD *thd, bool Castagnoli, Item *a) :
+    Item_long_func(thd, a),
+    crc_func(Castagnoli ? my_crc32c : my_checksum)
+  { unsigned_flag= 1; }
+  Item_func_crc32(THD *thd, bool Castagnoli, Item *a, Item *b) :
+    Item_long_func(thd, a, b),
+    crc_func(Castagnoli ? my_crc32c : my_checksum)
   { unsigned_flag= 1; }
   LEX_CSTRING func_name_cstring() const override
   {
-    static LEX_CSTRING name= {STRING_WITH_LEN("crc32") };
-    return name;
+    static LEX_CSTRING crc32_name= {STRING_WITH_LEN("crc32") };
+    static LEX_CSTRING crc32c_name= {STRING_WITH_LEN("crc32c") };
+    return crc_func == my_crc32c ? crc32c_name : crc32_name;
   }
-  bool fix_length_and_dec() override { max_length=10; return FALSE; }
+  bool fix_length_and_dec(THD *thd) override { max_length=10; return FALSE; }
   longlong val_int() override;
   Item *get_copy(THD *thd) override
   { return get_item_copy<Item_func_crc32>(thd, this); }
@@ -1972,7 +1983,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("uncompressed_length") };
     return name;
   }
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length=10;
     set_maybe_null();
@@ -1994,7 +2005,7 @@ class Item_func_compress: public Item_str_binary_checksum_func
 public:
   Item_func_compress(THD *thd, Item *a)
    :Item_str_binary_checksum_func(thd, a) {}
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= (args[0]->max_length * 120) / 100 + 12;
     return FALSE;
@@ -2015,7 +2026,7 @@ class Item_func_uncompress: public Item_str_binary_checksum_func
 public:
   Item_func_uncompress(THD *thd, Item *a)
    :Item_str_binary_checksum_func(thd, a) {}
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     set_maybe_null();
     max_length= MAX_BLOB_WIDTH;
@@ -2045,7 +2056,7 @@ protected:
 public:
   Item_func_dyncol_create(THD *thd, List<Item> &args, DYNCALL_CREATE_DEF *dfs);
   bool fix_fields(THD *thd, Item **ref) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   LEX_CSTRING func_name_cstring() const override
   {
     static LEX_CSTRING name= {STRING_WITH_LEN("column_create") };
@@ -2087,7 +2098,7 @@ public:
     return name;
   }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= MAX_BLOB_WIDTH;
     set_maybe_null();
@@ -2107,7 +2118,7 @@ class Item_dyncol_get: public Item_str_func
 public:
   Item_dyncol_get(THD *thd, Item *str, Item *num): Item_str_func(thd, str, num)
   {}
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     set_maybe_null();
     max_length= MAX_BLOB_WIDTH;
@@ -2154,7 +2165,7 @@ class Item_func_dyncol_list: public Item_str_func
 public:
   Item_func_dyncol_list(THD *thd, Item *str): Item_str_func(thd, str)
     {collation.set(DYNCOL_UTF);}
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     set_maybe_null();
     max_length= MAX_BLOB_WIDTH;
@@ -2191,7 +2202,7 @@ public:
     static LEX_CSTRING name= {STRING_WITH_LEN("<rowid>") };
     return name;
   }
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   Item *get_copy(THD *thd) override
   { return get_item_copy<Item_temptable_rowid>(thd, this); }
 };
@@ -2210,7 +2221,7 @@ public:
     return name;
   }
   String *val_str_ascii(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= WSREP_GTID_STR_LEN;
     set_maybe_null();
@@ -2231,7 +2242,7 @@ public:
     return name;
   }
   String *val_str_ascii(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
     max_length= WSREP_GTID_STR_LEN;
     set_maybe_null();
