@@ -1,5 +1,5 @@
-/* Copyright (C) 2008-2019 Kentoku Shiba
-   Copyright (C) 2019 MariaDB corp
+/* Copyright (C) 2008-2020 Kentoku Shiba
+   Copyright (C) 2019-2020 MariaDB corp
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,9 +22,25 @@
 #define SPIDER_SIMPLE_CONNECT             1
 #define SPIDER_SIMPLE_DISCONNECT          2
 #define SPIDER_SIMPLE_RECORDS             3
-#ifdef HA_HAS_CHECKSUM_EXTENDED
 #define SPIDER_SIMPLE_CHECKSUM_TABLE      4
-#endif
+
+#define SPIDER_LOP_CHK_QUEUED             (1 << 0)
+#define SPIDER_LOP_CHK_MERAGED            (1 << 1)
+#define SPIDER_LOP_CHK_IGNORED            (1 << 2)
+
+typedef struct st_spider_conn_loop_check
+{
+  uint               flag;
+  my_hash_value_type hash_value_to;
+  my_hash_value_type hash_value_full;
+  LEX_CSTRING        from_name;
+  LEX_CSTRING        cur_name;
+  LEX_CSTRING        to_name;
+  LEX_CSTRING        full_name;
+  LEX_CSTRING        from_value;
+  LEX_CSTRING        merged_value;
+  st_spider_conn_loop_check *next;
+} SPIDER_CONN_LOOP_CHECK;
 
 uchar *spider_conn_get_key(
   SPIDER_CONN *conn,
@@ -38,6 +54,14 @@ uchar *spider_ipport_conn_get_key(
   my_bool not_used __attribute__ ((unused))
 );
 
+int spider_conn_init(
+  SPIDER_CONN *conn
+);
+
+void spider_conn_done(
+  SPIDER_CONN *conn
+);
+
 int spider_reset_conn_setted_parameter(
   SPIDER_CONN *conn,
   THD *thd
@@ -155,6 +179,21 @@ void spider_conn_queue_UTC_time_zone(
   SPIDER_CONN *conn
 );
 
+int spider_conn_queue_and_merge_loop_check(
+  SPIDER_CONN *conn,
+  SPIDER_CONN_LOOP_CHECK *lcptr
+);
+
+int spider_conn_reset_queue_loop_check(
+  SPIDER_CONN *conn
+);
+
+int spider_conn_queue_loop_check(
+  SPIDER_CONN *conn,
+  ha_spider *spider,
+  int link_idx
+);
+
 void spider_conn_queue_start_transaction(
   SPIDER_CONN *conn
 );
@@ -213,7 +252,6 @@ SPIDER_CONN *spider_tree_delete(
   SPIDER_CONN *top
 );
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 int spider_set_conn_bg_param(
   ha_spider *spider
 );
@@ -310,7 +348,6 @@ void spider_free_mon_threads(
 void *spider_bg_mon_action(
   void *arg
 );
-#endif
 
 int spider_conn_first_link_idx(
   THD *thd,
