@@ -22,33 +22,37 @@ public:
     spider_string *str,
     const char *name,
     uint name_length
-  ) override;
+  );
   int append_name_with_charset(
     spider_string *str,
     const char *name,
     uint name_length,
     CHARSET_INFO *name_charset
-  ) override;
+  );
   int append_escaped_name(
     spider_string *str,
     const char *name,
     uint name_length
-  ) override;
+  );
   int append_escaped_name_with_charset(
     spider_string *str,
     const char *name,
     uint name_length,
     CHARSET_INFO *name_charset
-  ) override;
+  );
   bool is_name_quote(
     const char head_code
-  ) override;
+  );
   int append_escaped_name_quote(
     spider_string *str
-  ) override;
-  int append_column_value(ha_spider *spider, spider_string *str, Field *field,
-                          const uchar *new_ptr, bool is_like,
-                          CHARSET_INFO *access_charset) override;
+  );
+  int append_column_value(
+    ha_spider *spider,
+    spider_string *str,
+    Field *field,
+    const uchar *new_ptr,
+    CHARSET_INFO *access_charset
+  );
   int append_from_with_alias(
     spider_string *str,
     const char **table_names,
@@ -62,19 +66,19 @@ public:
   int append_trx_isolation(
     spider_string *str,
     int trx_isolation
-  ) override;
+  );
   int append_autocommit(
     spider_string *str,
     bool autocommit
-  ) override;
+  );
   int append_sql_log_off(
     spider_string *str,
     bool sql_log_off
-  ) override;
+  );
   int append_wait_timeout(
     spider_string *str,
     int wait_timeout
-  ) override;
+  );
   virtual int append_sql_mode_internal(
     spider_string *str,
     sql_mode_t sql_mode
@@ -82,25 +86,25 @@ public:
   int append_sql_mode(
     spider_string *str,
     sql_mode_t sql_mode
-  ) override;
+  );
   int append_time_zone(
     spider_string *str,
     Time_zone *time_zone
-  ) override;
+  );
   int append_loop_check(
     spider_string *str,
     SPIDER_CONN *conn
-  ) override;
+  );
   int append_start_transaction(
     spider_string *str
-  ) override;
+  );
   int append_xa_start(
     spider_string *str,
     XID *xid
-  ) override;
+  );
   int append_lock_table_head(
     spider_string *str
-  ) override;
+  );
   int append_lock_table_body(
     spider_string *str,
     const char *db_name,
@@ -110,13 +114,13 @@ public:
     uint table_name_length,
     CHARSET_INFO *table_name_charset,
     int lock_type
-  ) override;
+  );
   int append_lock_table_tail(
     spider_string *str
-  ) override;
+  );
   int append_unlock_table(
     spider_string *str
-  ) override;
+  );
   int open_item_func(
     Item_func *item_func,
     ha_spider *spider,
@@ -125,8 +129,7 @@ public:
     uint alias_length,
     bool use_fields,
     spider_fields *fields
-  ) override;
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
+  );
   int open_item_sum_func(
     Item_sum *item_sum,
     ha_spider *spider,
@@ -135,13 +138,11 @@ public:
     uint alias_length,
     bool use_fields,
     spider_fields *fields
-  ) override;
-#endif
+  );
   int append_escaped_util(
     spider_string *to,
     String *from
-  ) override;
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
+  );
   int append_table(
     ha_spider *spider,
     spider_fields *fields,
@@ -182,19 +183,18 @@ public:
     spider_string *str,
     TABLE_LIST *table_list,
     uint table_count
-  ) override;
+  );
   int reappend_tables(
     spider_fields *fields,
     SPIDER_LINK_IDX_CHAIN *link_idx_chain,
     spider_string *str
-  ) override;
+  );
   int append_where(
     spider_string *str
-  ) override;
+  );
   int append_having(
     spider_string *str
-  ) override;
-#endif
+  );
   bool tables_on_different_db_are_joinable();
   bool socket_has_default_value();
   bool database_has_default_value();
@@ -214,9 +214,8 @@ public:
     spider_string *str,
     Field *field,
     const uchar *new_ptr,
-    bool is_like,
     CHARSET_INFO *access_charset
-  ) override;
+  );
 };
 
 class spider_db_mariadb_util: public spider_db_mbase_util
@@ -227,15 +226,14 @@ public:
   int append_sql_mode_internal(
     spider_string *str,
     sql_mode_t sql_mode
-  ) override;
+  );
   int append_column_value(
     ha_spider *spider,
     spider_string *str,
     Field *field,
     const uchar *new_ptr,
-    bool is_like,
     CHARSET_INFO *access_charset
-  ) override;
+  );
 };
 
 class spider_db_mbase_row: public spider_db_row
@@ -328,11 +326,9 @@ public:
     int mode,
     ha_rows &records
   );
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   int fetch_table_checksum(
     ha_spider *spider
   );
-#endif
   int fetch_table_cardinality(
     int mode,
     TABLE *table,
@@ -356,7 +352,6 @@ public:
     longlong pos
   );
   int get_errno();
-#ifdef SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
   int fetch_columns_for_discover_table_structure(
     spider_string *str,
     CHARSET_INFO *access_charset
@@ -370,7 +365,6 @@ public:
     SPIDER_SHARE *spider_share,
     CHARSET_INFO *access_charset
   );
-#endif
 };
 
 class spider_db_mysql_result: public spider_db_mbase_result
@@ -636,9 +630,7 @@ public:
   spider_string      *table_names_str;
   spider_string      *db_names_str;
   spider_string      *db_table_str;
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   my_hash_value_type *db_table_str_hash_value;
-#endif
   uint               table_nm_max_length;
   uint               db_nm_max_length;
   spider_string      *column_name_str;
@@ -678,16 +670,12 @@ public:
     int *table_name_pos
   );
   bool need_change_db_table_name();
-#ifdef SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
   int discover_table_structure(
     SPIDER_TRX *trx,
     SPIDER_SHARE *spider_share,
     spider_string *str
   );
-#endif
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   bool checksum_support();
-#endif
 protected:
   int create_table_names_str();
   void free_table_names_str();
@@ -874,7 +862,6 @@ public:
   int append_update_set(
     spider_string *str
   );
-  #ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   int append_direct_update_set_part();
   int append_direct_update_set(
     spider_string *str
@@ -893,7 +880,6 @@ public:
     const char *alias,
     uint alias_length
   );
-  #endif
   int append_select_part(
     ulong sql_type
   );
@@ -1090,7 +1076,6 @@ public:
     const char *alias,
     uint alias_length
   );
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   int append_sum_select_part(
     ulong sql_type,
     const char *alias,
@@ -1101,14 +1086,12 @@ public:
     const char *alias,
     uint alias_length
   );
-#endif
   void set_order_pos(
     ulong sql_type
   );
   void set_order_to_pos(
     ulong sql_type
   );
-#ifdef HANDLER_HAS_DIRECT_AGGREGATE
   int append_group_by_part(
     const char *alias,
     uint alias_length,
@@ -1119,7 +1102,6 @@ public:
     const char *alias,
     uint alias_length
   );
-#endif
   int append_key_order_for_merge_with_alias_part(
     const char *alias,
     uint alias_length,
@@ -1403,13 +1385,11 @@ public:
   bool need_lock_before_set_sql_for_exec(
     ulong sql_type
   );
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   int set_sql_for_exec(
     ulong sql_type,
     int link_idx,
     SPIDER_LINK_IDX_CHAIN *link_idx_chain
   );
-#endif
   int set_sql_for_exec(
     ulong sql_type,
     int link_idx
@@ -1447,11 +1427,9 @@ public:
   int show_records(
     int link_idx
   );
-#ifdef HA_HAS_CHECKSUM_EXTENDED
   int checksum_table(
     int link_idx
   );
-#endif
   int show_last_insert_id(
     int link_idx,
     ulonglong &last_insert_id
@@ -1530,7 +1508,6 @@ public:
     int link_idx,
     ulong sql_type
   );
-#ifdef SPIDER_HAS_GROUP_BY_HANDLER
   int append_from_and_tables_part(
     spider_fields *fields,
     ulong sql_type
@@ -1601,8 +1578,6 @@ public:
     bool use_fields,
     spider_fields *fields
   );
-#endif
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
   bool check_direct_update(
     st_select_lex *select_lex,
     longlong select_limit,
@@ -1613,7 +1588,6 @@ public:
     longlong select_limit,
     longlong offset_limit
   );
-#endif
 };
 
 class spider_mysql_handler: public spider_mbase_handler
