@@ -1,4 +1,5 @@
-/* Copyright (C) 2009-2018 Kentoku Shiba
+/* Copyright (C) 2009-2019 Kentoku Shiba
+   Copyright (C) 2019 MariaDB corp
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -17,16 +18,11 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
 #include "sql_partition.h"
 #include "sql_acl.h"
-#endif
 #include "spd_err.h"
 #include "spd_param.h"
 #include "spd_db_include.h"
@@ -87,9 +83,7 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_mon_list(
   SPIDER_TABLE_MON_LIST *table_mon_list;
   MEM_ROOT mem_root;
   ulonglong mon_table_cache_version;
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   my_hash_value_type hash_value;
-#endif
   DBUG_ENTER("spider_get_ping_table_mon_list");
   if (spider_mon_table_cache_version != spider_mon_table_cache_version_req)
   {
@@ -107,27 +101,17 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_mon_list(
     spider_param_udf_table_mon_mutex_count());
   DBUG_PRINT("info",("spider hash key=%s", str->c_ptr()));
   DBUG_PRINT("info",("spider hash key length=%u", str->length()));
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   hash_value = my_calc_hash(
     &spider_udf_table_mon_list_hash[mutex_hash],
     (uchar*) str->c_ptr(), str->length());
-#endif
   pthread_mutex_lock(&spider_udf_table_mon_mutexes[mutex_hash]);
   mon_table_cache_version = (ulonglong) spider_mon_table_cache_version;
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   if (!(table_mon_list = (SPIDER_TABLE_MON_LIST *)
     my_hash_search_using_hash_value(
       &spider_udf_table_mon_list_hash[mutex_hash], hash_value,
       (uchar*) str->c_ptr(), str->length())) ||
       table_mon_list->mon_table_cache_version != mon_table_cache_version
   )
-#else
-  if (!(table_mon_list = (SPIDER_TABLE_MON_LIST *) my_hash_search(
-    &spider_udf_table_mon_list_hash[mutex_hash],
-    (uchar*) str->c_ptr(), str->length())) ||
-    table_mon_list->mon_table_cache_version != mon_table_cache_version
-  )
-#endif
   {
     if (
       table_mon_list &&
@@ -146,17 +130,9 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_mon_list(
     table_mon_list->mon_table_cache_version = mon_table_cache_version;
     uint old_elements =
       spider_udf_table_mon_list_hash[mutex_hash].array.max_element;
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
     table_mon_list->key_hash_value = hash_value;
-#endif
-#ifdef HASH_UPDATE_WITH_HASH_VALUE
-    if (my_hash_insert_with_hash_value(
-      &spider_udf_table_mon_list_hash[mutex_hash],
-      hash_value, (uchar*) table_mon_list))
-#else
     if (my_hash_insert(&spider_udf_table_mon_list_hash[mutex_hash],
       (uchar*) table_mon_list))
-#endif
     {
       spider_ping_table_free_mon_list(table_mon_list);
       *error_num = HA_ERR_OUT_OF_MEM;
@@ -205,13 +181,8 @@ void spider_release_ping_table_mon_list_loop(
   SPIDER_TABLE_MON_LIST *table_mon_list
 ) {
   DBUG_ENTER("spider_release_ping_table_mon_list_loop");
-#ifdef HASH_UPDATE_WITH_HASH_VALUE
-  my_hash_delete_with_hash_value(&spider_udf_table_mon_list_hash[mutex_hash],
-    table_mon_list->key_hash_value, (uchar*) table_mon_list);
-#else
   my_hash_delete(&spider_udf_table_mon_list_hash[mutex_hash],
     (uchar*) table_mon_list);
-#endif
   while (TRUE)
   {
     if (table_mon_list->use_count)
@@ -255,22 +226,14 @@ int spider_release_ping_table_mon_list(
 
   mutex_hash = spider_udf_calc_hash(conv_name_str.c_ptr_safe(),
     spider_param_udf_table_mon_mutex_count());
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   my_hash_value_type hash_value = my_calc_hash(
     &spider_udf_table_mon_list_hash[mutex_hash],
     (uchar*) conv_name_str.c_ptr(), conv_name_str.length());
-#endif
   pthread_mutex_lock(&spider_udf_table_mon_mutexes[mutex_hash]);
-#ifdef SPIDER_HAS_HASH_VALUE_TYPE
   if ((table_mon_list = (SPIDER_TABLE_MON_LIST *)
     my_hash_search_using_hash_value(
       &spider_udf_table_mon_list_hash[mutex_hash], hash_value,
       (uchar*) conv_name_str.c_ptr(), conv_name_str.length())))
-#else
-  if ((table_mon_list = (SPIDER_TABLE_MON_LIST *) my_hash_search(
-    &spider_udf_table_mon_list_hash[mutex_hash],
-    (uchar*) conv_name_str.c_ptr(), conv_name_str.length())))
-#endif
     spider_release_ping_table_mon_list_loop(mutex_hash, table_mon_list);
   pthread_mutex_unlock(&spider_udf_table_mon_mutexes[mutex_hash]);
   my_afree(buf);
@@ -289,11 +252,7 @@ int spider_get_ping_table_mon(
 ) {
   int error_num;
   TABLE *table_link_mon = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
+  SPIDER_Open_tables_backup open_tables_backup;
   char table_key[MAX_KEY_LENGTH];
   SPIDER_TABLE_MON *table_mon, *table_mon_prev = NULL;
   SPIDER_SHARE *tmp_share;
@@ -367,12 +326,15 @@ create_table_mon:
   do {
     if (!(table_mon = (SPIDER_TABLE_MON *)
       spider_bulk_malloc(spider_current_trx, 35, MYF(MY_WME | MY_ZEROFILL),
-        &table_mon, sizeof(SPIDER_TABLE_MON),
-        &tmp_share, sizeof(SPIDER_SHARE),
-        &tmp_connect_info, sizeof(char *) * SPIDER_TMP_SHARE_CHAR_PTR_COUNT,
-        &tmp_connect_info_length, sizeof(uint) * SPIDER_TMP_SHARE_UINT_COUNT,
-        &tmp_long, sizeof(long) * SPIDER_TMP_SHARE_LONG_COUNT,
-        &tmp_longlong, sizeof(longlong) * SPIDER_TMP_SHARE_LONGLONG_COUNT,
+        &table_mon, (uint) (sizeof(SPIDER_TABLE_MON)),
+        &tmp_share, (uint) (sizeof(SPIDER_SHARE)),
+        &tmp_connect_info,
+          (uint) (sizeof(char *) * SPIDER_TMP_SHARE_CHAR_PTR_COUNT),
+        &tmp_connect_info_length,
+          (uint) (sizeof(uint) * SPIDER_TMP_SHARE_UINT_COUNT),
+        &tmp_long, (uint) (sizeof(long) * SPIDER_TMP_SHARE_LONG_COUNT),
+        &tmp_longlong,
+          (uint) (sizeof(longlong) * SPIDER_TMP_SHARE_LONGLONG_COUNT),
         NullS))
     ) {
       spider_sys_index_end(table_link_mon);
@@ -403,10 +365,8 @@ create_table_mon:
     if (
       (error_num = spider_set_connect_info_default(
         tmp_share,
-#ifdef WITH_PARTITION_STORAGE_ENGINE
         NULL,
         NULL,
-#endif
         NULL
       )) ||
       (error_num = spider_set_connect_info_default_dbtable(
@@ -471,11 +431,7 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
   int *error_num
 ) {
   TABLE *table_tables = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
+  SPIDER_Open_tables_backup open_tables_backup;
   char table_key[MAX_KEY_LENGTH];
 
   SPIDER_TABLE_MON_LIST *table_mon_list = NULL;
@@ -491,13 +447,17 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
   SPD_INIT_ALLOC_ROOT(&mem_root, 4096, 0, MYF(MY_WME));
   if (!(table_mon_list = (SPIDER_TABLE_MON_LIST *)
     spider_bulk_malloc(spider_current_trx, 36, MYF(MY_WME | MY_ZEROFILL),
-      &table_mon_list, sizeof(SPIDER_TABLE_MON_LIST),
-      &tmp_share, sizeof(SPIDER_SHARE),
-      &tmp_connect_info, sizeof(char *) * SPIDER_TMP_SHARE_CHAR_PTR_COUNT,
-      &tmp_connect_info_length, sizeof(uint) * SPIDER_TMP_SHARE_UINT_COUNT,
-      &tmp_long, sizeof(long) * SPIDER_TMP_SHARE_LONG_COUNT,
-      &tmp_longlong, sizeof(longlong) * SPIDER_TMP_SHARE_LONGLONG_COUNT,
-      &key_str, str->length() + 1,
+      &table_mon_list, (uint) (sizeof(SPIDER_TABLE_MON_LIST)),
+      &tmp_share, (uint) (sizeof(SPIDER_SHARE)),
+      &tmp_connect_info,
+        (uint) (sizeof(char *) * SPIDER_TMP_SHARE_CHAR_PTR_COUNT),
+      &tmp_connect_info_length,
+        (uint) (sizeof(uint) * SPIDER_TMP_SHARE_UINT_COUNT),
+      &tmp_long,
+        (uint) (sizeof(long) * SPIDER_TMP_SHARE_LONG_COUNT),
+      &tmp_longlong,
+        (uint) (sizeof(longlong) * SPIDER_TMP_SHARE_LONGLONG_COUNT),
+      &key_str, (uint) (str->length() + 1),
       NullS))
   ) {
     my_error(HA_ERR_OUT_OF_MEM, MYF(0));
@@ -559,10 +519,8 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
   if (
     (*error_num = spider_set_connect_info_default(
       tmp_share,
-#ifdef WITH_PARTITION_STORAGE_ENGINE
       NULL,
       NULL,
-#endif
       NULL
     )) ||
     (*error_num = spider_set_connect_info_default_dbtable(
@@ -581,43 +539,26 @@ SPIDER_TABLE_MON_LIST *spider_get_ping_table_tgt(
   if (tmp_share->link_statuses[0] == SPIDER_LINK_STATUS_NG)
     table_mon_list->mon_status = SPIDER_LINK_MON_NG;
 
-#if MYSQL_VERSION_ID < 50500
-  if (pthread_mutex_init(&table_mon_list->caller_mutex, MY_MUTEX_INIT_FAST))
-#else
   if (mysql_mutex_init(spd_key_mutex_mon_list_caller,
     &table_mon_list->caller_mutex, MY_MUTEX_INIT_FAST))
-#endif
   {
     *error_num = HA_ERR_OUT_OF_MEM;
     goto error_caller_mutex_init;
   }
-#if MYSQL_VERSION_ID < 50500
-  if (pthread_mutex_init(&table_mon_list->receptor_mutex, MY_MUTEX_INIT_FAST))
-#else
   if (mysql_mutex_init(spd_key_mutex_mon_list_receptor,
     &table_mon_list->receptor_mutex, MY_MUTEX_INIT_FAST))
-#endif
   {
     *error_num = HA_ERR_OUT_OF_MEM;
     goto error_receptor_mutex_init;
   }
-#if MYSQL_VERSION_ID < 50500
-  if (pthread_mutex_init(&table_mon_list->monitor_mutex, MY_MUTEX_INIT_FAST))
-#else
   if (mysql_mutex_init(spd_key_mutex_mon_list_monitor,
     &table_mon_list->monitor_mutex, MY_MUTEX_INIT_FAST))
-#endif
   {
     *error_num = HA_ERR_OUT_OF_MEM;
     goto error_monitor_mutex_init;
   }
-#if MYSQL_VERSION_ID < 50500
-  if (pthread_mutex_init(&table_mon_list->update_status_mutex,
-    MY_MUTEX_INIT_FAST))
-#else
   if (mysql_mutex_init(spd_key_mutex_mon_list_update_status,
     &table_mon_list->update_status_mutex, MY_MUTEX_INIT_FAST))
-#endif
   {
     *error_num = HA_ERR_OUT_OF_MEM;
     goto error_update_status_mutex_init;
@@ -690,12 +631,9 @@ int spider_get_ping_table_gtid_pos(
   int error_num, source_link_idx, need_mon;
   char table_key[MAX_KEY_LENGTH];
   TABLE *table_tables, *table_gtid_pos;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup_tables;
-  Open_tables_state open_tables_backup_gtid_pos;
-#else
-  Open_tables_backup open_tables_backup_tables;
-  Open_tables_backup open_tables_backup_gtid_pos;
+  SPIDER_Open_tables_backup open_tables_backup_tables;
+#ifdef SPIDER_REQUIRE_DEFINE_FOR_SECONDARY_OPEN_TABLES_BACKUP
+  SPIDER_Open_tables_backup open_tables_backup_gtid_pos;
 #endif
   MEM_ROOT mem_root;
   long link_status;
@@ -713,6 +651,7 @@ int spider_get_ping_table_gtid_pos(
       db_name = setted db_name and
       table_name = setted table_name
   */
+#ifdef SPIDER_REQUIRE_DEFINE_FOR_SECONDARY_OPEN_TABLES_BACKUP
   if (
     !(table_tables = spider_open_sys_table(
       thd, SPIDER_SYS_TABLES_TABLE_NAME_STR,
@@ -728,6 +667,45 @@ int spider_get_ping_table_gtid_pos(
       &open_tables_backup_gtid_pos, need_lock, &error_num))
   )
     goto error_open_table_gtid_pos;
+#else
+  TABLE_LIST tables_tables;
+  TABLE_LIST tables_gtid_pos;
+  TABLE_LIST *tables = &tables_tables;
+  LEX_CSTRING db_name =
+  {
+    "mysql",
+    sizeof("mysql") - 1
+  };
+  LEX_CSTRING tbl_name_tables =
+  {
+    SPIDER_SYS_TABLES_TABLE_NAME_STR,
+    SPIDER_SYS_TABLES_TABLE_NAME_LEN
+  };
+  LEX_CSTRING tbl_name_gtid_pos =
+  {
+    SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_STR,
+    SPIDER_SYS_POS_FOR_RECOVERY_TABLE_NAME_LEN
+  };
+  tables_tables.init_one_table(&db_name, &tbl_name_tables, 0, TL_READ);
+  tables_gtid_pos.init_one_table(&db_name, &tbl_name_gtid_pos, 0, TL_READ);
+  MDL_REQUEST_INIT(&tables_tables.mdl_request, MDL_key::TABLE,
+    SPIDER_TABLE_LIST_db_str(&tables_tables),
+    SPIDER_TABLE_LIST_table_name_str(&tables_tables),
+    MDL_SHARED_READ, MDL_TRANSACTION);
+  MDL_REQUEST_INIT(&tables_gtid_pos.mdl_request, MDL_key::TABLE,
+    SPIDER_TABLE_LIST_db_str(&tables_gtid_pos),
+    SPIDER_TABLE_LIST_table_name_str(&tables_gtid_pos),
+    MDL_SHARED_READ, MDL_TRANSACTION);
+  tables_tables.next_global = &tables_gtid_pos;
+  if (spider_sys_open_and_lock_tables(thd, &tables,
+    &open_tables_backup_tables))
+  {
+    error_num = my_errno;
+    goto error_open_table_tables;
+  }
+  table_tables = tables_tables.table;
+  table_gtid_pos = tables_gtid_pos.table;
+#endif
 
   table_tables->use_all_columns();
   table_gtid_pos->use_all_columns();
@@ -814,8 +792,10 @@ int spider_get_ping_table_gtid_pos(
   {
     goto error_sys_index_end;
   }
-  spider_close_sys_table(thd, table_gtid_pos, &open_tables_backup_gtid_pos,
-    need_lock);
+#ifdef SPIDER_REQUIRE_DEFINE_FOR_SECONDARY_OPEN_TABLES_BACKUP
+  spider_close_sys_table(thd, table_gtid_pos,
+    &open_tables_backup_gtid_pos, need_lock);
+#endif
   spider_close_sys_table(thd, table_tables, &open_tables_backup_tables,
     need_lock);
 
@@ -827,9 +807,12 @@ error_get_sys_tables_link_status:
   spider_sys_index_end(table_tables);
 error_sys_index_end:
 error_get_sys_table_by_idx:
-  spider_close_sys_table(thd, table_gtid_pos, &open_tables_backup_gtid_pos,
+#ifdef SPIDER_REQUIRE_DEFINE_FOR_SECONDARY_OPEN_TABLES_BACKUP
+  spider_close_sys_table(thd, table_gtid_pos,
+    &open_tables_backup_gtid_pos,
     need_lock);
 error_open_table_gtid_pos:
+#endif
   spider_close_sys_table(thd, table_tables, &open_tables_backup_tables,
     need_lock);
 error_open_table_tables:
@@ -842,12 +825,9 @@ int spider_init_ping_table_mon_cache(
   bool need_lock
 ) {
   int error_num, same;
+  uint old_elements;
   TABLE *table_link_mon = NULL;
-#if MYSQL_VERSION_ID < 50500
-  Open_tables_state open_tables_backup;
-#else
-  Open_tables_backup open_tables_backup;
-#endif
+  SPIDER_Open_tables_backup open_tables_backup;
   SPIDER_MON_KEY mon_key;
   DBUG_ENTER("spider_init_ping_table_mon_cache");
 
@@ -891,11 +871,23 @@ int spider_init_ping_table_mon_cache(
         {
           mon_key.sort = spider_calc_for_sort(3, mon_key.db_name,
             mon_key.table_name, mon_key.link_id);
+          old_elements = spider_mon_table_cache.max_element;
           if (push_dynamic(&spider_mon_table_cache, (uchar *) &mon_key))
           {
             error_num = HA_ERR_OUT_OF_MEM;
             goto error_push_dynamic;
           }
+          if (spider_mon_table_cache.max_element != old_elements)
+          {
+            spider_free_mem_calc(spider_current_trx,
+              spider_mon_table_cache_id,
+              old_elements *
+              spider_mon_table_cache.size_of_element);
+            spider_alloc_calc_mem(spider_current_trx,
+              spider_mon_table_cache,
+              spider_mon_table_cache.max_element *
+              spider_mon_table_cache.size_of_element);
+          }
         }
 
         if ((error_num = spider_sys_index_next(table_link_mon)))
@@ -915,12 +907,16 @@ int spider_init_ping_table_mon_cache(
       (uchar *) dynamic_element(&spider_mon_table_cache, 0, SPIDER_MON_KEY *),
       spider_mon_table_cache.elements, sizeof(SPIDER_MON_KEY),
       (qsort_cmp) spider_compare_for_sort);
-    uint old_elements = spider_mon_table_cache.max_element;
+    old_elements = spider_mon_table_cache.max_element;
     freeze_size(&spider_mon_table_cache);
-    if (spider_mon_table_cache.max_element < old_elements)
+    if (spider_mon_table_cache.max_element != old_elements)
     {
       spider_free_mem_calc(spider_current_trx,
         spider_mon_table_cache_id,
+        old_elements *
+        spider_mon_table_cache.size_of_element);
+      spider_alloc_calc_mem(spider_current_trx,
+        spider_mon_table_cache,
         spider_mon_table_cache.max_element *
         spider_mon_table_cache.size_of_element);
     }
@@ -1030,23 +1026,14 @@ long long spider_ping_table_body(
   conv_name.init_calc_mem(135);
   tmp_str.init_calc_mem(247);
   conv_name.length(0);
-#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100002
   server_id = global_system_variables.server_id;
-#else
-  server_id = thd->server_id;
-#endif
   if (
     thd->open_tables != 0 ||
     thd->handler_tables_hash.records != 0 ||
     thd->derived_tables != 0 ||
     thd->lock != 0 ||
-#if MYSQL_VERSION_ID < 50500
-    thd->locked_tables != 0 ||
-    thd->prelocked_mode != NON_PRELOCKED
-#else
     thd->locked_tables_list.locked_tables() ||
     thd->locked_tables_mode != LTM_NONE
-#endif
   ) {
     if (thd->open_tables != 0)
     {
@@ -1069,18 +1056,6 @@ long long spider_ping_table_body(
       my_printf_error(ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_NUM,
         ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_STR_WITH_PTR, MYF(0),
         "thd->lock", thd->lock);
-#if MYSQL_VERSION_ID < 50500
-    } else if (thd->locked_tables != 0)
-    {
-      my_printf_error(ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_NUM,
-        ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_STR_WITH_PTR, MYF(0),
-        "thd->locked_tables", thd->locked_tables);
-    } else if (thd->prelocked_mode != NON_PRELOCKED)
-    {
-      my_printf_error(ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_NUM,
-        ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_STR_WITH_NUM, MYF(0),
-        "thd->prelocked_mode", (longlong) thd->prelocked_mode);
-#else
     } else if (thd->locked_tables_list.locked_tables())
     {
       my_printf_error(ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_NUM,
@@ -1092,7 +1067,6 @@ long long spider_ping_table_body(
       my_printf_error(ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_NUM,
         ER_SPIDER_UDF_CANT_USE_IF_OPEN_TABLE_STR_WITH_NUM, MYF(0),
         "thd->locked_tables_mode", (longlong) thd->locked_tables_mode);
-#endif
     }
     goto error;
   }
